/* queue.c
**
**    create the billard-queue display lists
**    Copyright (C) 2001  Florian Berger
**    Email:  harpin_floh@yahoo.de,  florian.berger@jk.uni-linz.ac.at
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License Version 2 as
**    published by the Free Software Foundation;
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

#include <GL/glu.h>
#include <math.h>
#include <stdlib.h>
#include "billard.h"
#include "queue.h"
#include "png_loader.h"
#include "options.h"


int create_queue()
{
    int queue_obj, i;
    int sidenr=16;
    GLfloat dl1=0.004;
    GLfloat dl2=0.001;
    double ph;
    VMvect v1,v2,n;



   queue_obj = glGenLists(1);
   glNewList(queue_obj, GL_COMPILE);

   glBegin(GL_QUAD_STRIP);
   for(i=0;i<sidenr+1;i++){
       ph=2.0*M_PI/(double)sidenr*i;
       v1.x = QUEUE_D1/2.0*cos(ph);
       v1.y = QUEUE_D1/2.0*sin(ph);
       v1.z = QUEUE_L-dl1;
       v2.x = QUEUE_D2/2.0*cos(ph);
       v2.y = QUEUE_D2/2.0*sin(ph);
       v2.z = dl2;
       n.x = (QUEUE_L-dl1-dl2)*cos(ph);
       n.y = (QUEUE_L-dl1-dl2)*sin(ph);
       n.z = -(QUEUE_D1-QUEUE_D2)/2.0;
       n=vec_unit(n);
       glNormal3f( n.x,  n.y,  n.z  );
       glTexCoord2f( 1.0-v2.z/QUEUE_L, (v2.x+QUEUE_D1/2.0)/QUEUE_D1 );
       glVertex3f( v2.x, v2.y, v2.z );
       glNormal3f( n.x,  n.y,  0.0  );
       glTexCoord2f( 1.0-v1.z/QUEUE_L, (v1.x+QUEUE_D1/2.0)/QUEUE_D1 );
       glVertex3f( v1.x, v1.y, v1.z );
   }
   glEnd();

   glBegin(GL_TRIANGLE_FAN);
   glNormal3f( 0.0,  0.0, -1.0  );
   glVertex3f( 0.0,  0.0,  0.0  );
   for(i=0;i<sidenr+1;i++){
       ph=2.0*M_PI/(double)sidenr*i;
       v2.x = QUEUE_D2/2.0*cos(ph);
       v2.y = QUEUE_D2/2.0*sin(ph);
       v2.z = dl2;
       glTexCoord2f( 1.0-v2.z/QUEUE_L, (v2.x+QUEUE_D1/2.0)/QUEUE_D1 );
       glVertex3f( v2.x, v2.y, v2.z );
   }
   glEnd();

   glBegin(GL_TRIANGLE_FAN);
   glNormal3f( 0.0,  0.0,  1.0  );
   glVertex3f( 0.0,  0.0,  QUEUE_L );
   for(i=0;i<sidenr+1;i++){
       ph=2.0*M_PI-2.0*M_PI/(double)sidenr*i;
       v1.x = QUEUE_D1/2.0*cos(ph);
       v1.y = QUEUE_D1/2.0*sin(ph);
       v1.z = QUEUE_L-dl1;
       glTexCoord2f( 1.0-v1.z/QUEUE_L, (v1.x+QUEUE_D1/2.0)/QUEUE_D1 );
       glVertex3f( v1.x, v1.y, v1.z );
   }
   glEnd();

   glEndList();

   return queue_obj;
}


void draw_queue( VMvect pos, GLfloat Xrot, GLfloat Zrot, GLfloat zoffs,
                 GLfloat xoffs, GLfloat yoffs, int spheretexbind )
{
    static int init=0;
    static int queue_obj;
    static int queuetexbind;
    static int queuetexw,queuetexh;
    static char * queuetexdata;
    GLfloat queue_col_spec[4] = {0.0, 0.0, 0.0, 1.0};
    GLfloat queue_col_diff[4] = {0.8, 0.8, 0.8, 1.0};
    GLfloat queue_col_refl[4] = {1.0, 1.0, 1.0, 0.2};
    GLfloat queue_col_amb [4];
    int i,depth;
    VMvect dir,nx,ny,hitpoint;

    dir = vec_xyz(sin(Zrot*M_PI/180.0)*sin(Xrot*M_PI/180.0),
                  cos(Zrot*M_PI/180.0)*sin(Xrot*M_PI/180.0),
                  cos(Xrot*M_PI/180.0));
    nx = vec_unit(vec_cross(vec_ez(),dir));  /* parallel to table */
    ny = vec_unit(vec_cross(nx,dir));        /* orthogonal to dir and nx */
    hitpoint = vec_add(vec_scale(nx,xoffs),vec_scale(ny,yoffs));
    pos = vec_add(pos,hitpoint);

    for(i=0;i<3;i++) queue_col_amb[i]=queue_col_diff[i]*0.5;

    if(!init){
        if( options_queue_tex ){
            glGenTextures(1,&queuetexbind);
            load_png("queue.png",&queuetexw,&queuetexh,&depth,&queuetexdata);
            glBindTexture(GL_TEXTURE_2D,queuetexbind);
            gluBuild2DMipmaps(GL_TEXTURE_2D, 3, queuetexw, queuetexh, GL_RGB,
                              GL_UNSIGNED_BYTE, queuetexdata);
            free(queuetexdata);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, options_tex_min_filter);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, options_tex_mag_filter);
        } else {
            queuetexbind=0;
        }
        queue_obj=create_queue();
        init=1;
    }

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glMaterialfv(GL_FRONT, GL_DIFFUSE,   queue_col_diff);
    glMaterialfv(GL_FRONT, GL_AMBIENT,   queue_col_amb);
    glMaterialfv(GL_FRONT, GL_SPECULAR,  queue_col_spec);
    glMaterialf (GL_FRONT, GL_SHININESS, 0.0 );

    glTranslatef( pos.x, pos.y, pos.z );
    glRotatef( -Zrot, 0.0, 0.0, 1.0 );
    glRotatef( -Xrot, 1.0, 0.0, 0.0 );
    glTranslatef( 0.0, 0.0, zoffs );
    glBindTexture(GL_TEXTURE_2D,queuetexbind);
    glCallList( queue_obj );

    /* draw queue-reflections */
    if( options_queue_reflect ){
        glDepthMask (GL_FALSE);
        glEnable(GL_BLEND);
        glBlendFunc (GL_SRC_ALPHA, GL_ONE);
        glMaterialfv(GL_FRONT, GL_DIFFUSE, queue_col_refl);
        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
        glBindTexture(GL_TEXTURE_2D,spheretexbind);
        glEnable(GL_TEXTURE_GEN_S);
        glEnable(GL_TEXTURE_GEN_T);
        glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
        glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
        glCallList( queue_obj );
        glDisable(GL_TEXTURE_GEN_S);
        glDisable(GL_TEXTURE_GEN_T);
        glDisable(GL_BLEND);
        glDepthMask (GL_TRUE);
    }
}
